import Action from '@overflow/commons/redux/Action';
import { ReducersMapObject } from 'redux';
import * as _ from 'lodash';

import Host from '../../api/model/Host';
import Port from '../../api/model/Port';
import Service from '../../api/model/Service';
import Zone from '../../api/model/Zone';

import * as IngActionTypes from '../action/ing';
import {
  HostState, hostDefaultState,
  PortState, portDefaultState,
  ServiceState, serviceDefaultState,
  DiscoveryIngState, DiscoveryIngDefaultState,
} from '../state/Ing';

const reducer: ReducersMapObject = {
  [IngActionTypes.RECEIVE_ZONE]: (state: DiscoveryIngState = DiscoveryIngDefaultState,
                                  action: Action<Zone>): DiscoveryIngState => {

    return {
      ...state,
    };
  },
  [IngActionTypes.RECEIVE_HOST]: (state: DiscoveryIngState = DiscoveryIngDefaultState,
                                  action: Action<Host>): DiscoveryIngState => {

    let obj: any = action.payload;
    let host: Host = JSON.parse(obj);
    let hostList: Host[] = null;
    if(state.hostList === null || state.hostList === undefined) {
      hostList = new Array();
      hostList.push(JSON.parse(obj));
    } else {

      hostList = _.clone(state.hostList);

      let tHost: Host = null;
      for(let h of hostList) {
        if(h.ip === host.ip) {
          tHost = h;
          break;
        }
      }

      if(tHost === null) {
        hostList.push(host);
      }
    }

    return {
      ...state,
      hostList: hostList,
    };
  },
  [IngActionTypes.RECEIVE_PORT]: (state: DiscoveryIngState = DiscoveryIngDefaultState,
                                  action: Action<Port>): DiscoveryIngState => {

    let hostList: Host[] = null;
    let obj: any = action.payload;
    if(state.hostList === null || state.hostList === undefined) {
      hostList = new Array();
      let port: Port = JSON.parse(obj);
      let host: Host = _.clone(port.host);
      host.ports = new Array();
      port.host = host;
      host.ports.push(port);
      hostList.push(port.host);

    } else {
      hostList = _.clone(state.hostList);
      let tHost: Host = null;
      let tPort: Port = null;
      let port: Port = JSON.parse(obj);

      for(let h of hostList) {
        if(h.ip === port.host.ip) {
          tHost = h;

          let chkObj: Object = h;
          if(chkObj.hasOwnProperty('ports') === false
        || h.ports === null) {
            let nPort: Port = _.clone(port);
            let nHost: Host = _.clone(h);
            nPort.host = nHost;
            nHost.ports = new Array();
            nHost.ports.push(nPort);
            hostList.push(nHost);
            break;
          }

          tPort = null;
          for(let p of h.ports) {
            if(p.portNumber === port.portNumber) {
              tPort = p;
              break;
            }
          }
          if(tPort === null) {
            let nHost: Host = _.clone(h);
            let nPort: Port = _.clone(port);
            nPort.host = nHost;

            // let dIdx: number = hostList.indexOf(h, 0);
            // if(dIdx > -1) {
            //   hostList.slice(dIdx, 1);
            // }
            hostList = removeArray(hostList, h);
            nHost.ports.push(nPort);
            hostList.push(nHost);
          }
        }
      }
      if(tHost === null) {
        let nHost: Host = _.clone(port.host);
        let nPort: Port = _.clone(port);

        nPort.host = nHost;
        nHost.ports = new Array();
        nHost.ports.push(nPort);
        hostList.push(nHost);
      }
    }


    return {
      ...state,
      hostList: hostList,
    };
  },
  [IngActionTypes.RECEIVE_SERVICE]: (state: DiscoveryIngState = DiscoveryIngDefaultState,
                                     action: Action<Service>): DiscoveryIngState => {

    let obj: any = action.payload;
    let service: Service = JSON.parse(obj);

    let hostList: Host[] = null;

    if(state.hostList === null || state.hostList === undefined) {
      hostList = new Array();
      let host: Host = _.clone(service.port.host);
      let port: Port = _.clone(service.port);
      port.host = host;
      service.port = port;
      port.services = new Array();
      port.services.push(service);
      host.ports = new Array();
      host.ports.push(port);

      hostList.push(host);
    } else {
      hostList = _.clone(state.hostList);
      let tHost: Host = null;
      let tPort: Port = null;
      let tService: Service = null;
      for(let h of hostList) {
        if(h.ip === service.port.host.ip) {
          tHost = h;

          let chkObj: Object = h;
          if(chkObj.hasOwnProperty('ports') === false
        || h.ports === null) {
            let nPort: Port = _.clone(service.port);
            let nHost: Host = _.clone(h);
            nPort.host = nHost;
            service.port = nPort;
            nHost.ports = new Array();
            nHost.ports.push(nPort);
            hostList.push(nHost);
            break;
          }

          tPort = null;
          for(let p of h.ports) {
            if(p.portNumber === service.port.portNumber) {
              tPort = p;

              let pChkObj: Object = p;
              if(pChkObj.hasOwnProperty('services') === false
            || p.services === null) {
                let nHost: Host = _.clone(h);
                let nPort: Port = _.clone(p);
                nPort.services = new Array();
                nPort.host = nHost;
                service.port = nPort;

                // let dIdx: number = hostList.indexOf(h, 0);
                // if(dIdx > -1) {
                //   hostList.slice(dIdx, 1);
                // }

                hostList = removeArray(hostList, h);

                // dIdx = nHost.ports.indexOf(p, 0);
                // if(dIdx > -1) {
                //   nHost.ports.slice(dIdx, 1);
                // }

                nHost.ports = removeArray(nHost.ports, p);

                nPort.services.push(service);
                nHost.ports.push(nPort);
                hostList.push(nHost);

                break;
              }


              tService = null;
              for(let s of p.services) {
                if(s.serviceName === service.serviceName) {
                 tService = null;
                 console.log('중복된 서비스 -' + service.serviceName);
                 return {
                   ...state,
                 };
                }

              }
              if(tService === null) {
                let nHost: Host = _.clone(h);
                let nPort: Port = _.clone(p);

                nPort.services.push(service);
                nHost.ports.push(nPort);
                hostList.push(nHost);
              }
            }

          }

          if(tPort === null) {
            let nHost: Host = _.clone(h);
            let nPort: Port = _.clone(service.port);
            nPort.services = new Array();
            nPort.host = nHost;
            service.port = nPort;

            // let dIdx: number = hostList.indexOf(h, 0);
            // if(dIdx > -1) {
            //   hostList.slice(dIdx, 1);
            // }

            hostList = removeArray(hostList, h);

            nPort.services.push(service);
            nHost.ports.push(nPort);
            hostList.push(nHost);
          }

          // service.port.host = h;
          // if(h.ports === null || h.ports === undefined) {
          //   h.ports = new Array();
          // }
          // h.ports.push(port);
        }

      }

      if(tHost === null) {
        let nHost: Host = _.clone(service.port.host);
        let nPort: Port = _.clone(service.port);

        nPort.host = nHost;
        service.port = nPort;

        nHost.ports = new Array();
        nPort.services = new Array();
        nPort.services.push(service);
        nHost.ports.push(nPort);
        hostList.push(nHost);
      }
    }

    return {
      ...state,
      hostList: hostList,
    };
  },
};

function removeArray(array: any[], obj: any): any[] {

  let dIdx: number = array.indexOf(obj, 0);
  if(dIdx > -1) {
    array.slice(dIdx, 1);
  }

  return array;
}

export default reducer;
