
import * as React from 'react';
import * as _ from 'lodash';
import { Icon, Step, Button, Table, Radio, Form, Container, Checkbox } from 'semantic-ui-react';
import { Grid, Image, Label, Segment, Dropdown, Input, List, Accordion, Loader, InputOnChangeData, CheckboxProps } from 'semantic-ui-react';

import MetaCrawler from '@overflow/meta/api/model/MetaCrawler';
import MetaCrawlerInputItem from '@overflow/meta/api/model/MetaCrawlerInputItem';

import SensorItemTree from '@overflow/meta/react/SensorItemTree';
import SensorRegistInfo from '@overflow/sensor/api/model/SensorRegistInfo';
import Infra from '@overflow/infra/api/model/Infra';
import AuthCrawler from '@overflow/auth/api/model/AuthCrawler';

export interface CrawlerAuthInputsStateProps {
  crawlerId?: number;
  crawler?: MetaCrawler;
  metaCrawlerInputItemList?: MetaCrawlerInputItem[];
  setSensor?(data: SensorRegistInfo): void;
  getSensor?(): SensorRegistInfo;
}

export interface CrawlerAuthInputsDispatchProps {
  onVerifyCrawler?(infraId: number, crawler: MetaCrawler, authJson: String): void;
  onRegistAuthCrawler?(authCrawler: AuthCrawler): void;
}

export type CrawlerAuthInputsProps = CrawlerAuthInputsStateProps & CrawlerAuthInputsDispatchProps;



export interface CrawlerAuthInputsState {
  isVerifying: boolean;
  authJson: any;
}

export class CrawlerAuthInputs extends React.Component<CrawlerAuthInputsProps, CrawlerAuthInputsState> {

  // private data: object[];
  constructor(props: any, context: any) {
    super(props, context);
    this.state = {
      isVerifying: false,
      authJson: {},
    };

  }

  public componentWillReceiveProps?(nextProps: Readonly<CrawlerAuthInputsProps>, nextContext: any): void {
    if (nextProps.metaCrawlerInputItemList !== this.props.metaCrawlerInputItemList) {
      this.setState({ authJson: {} });
    }
  }


  public componentWillMount(): void {
    // this.data = [
    //   {
    //     'id': '1',
    //     'metaInputType': {
    //       'id': '1',
    //       'name': 'Text',
    //     },
    //     'name': 'Community',
    //     'required': 'true',
    //     'defaultValue': 'public',
    //     'keyName': 'Community',
    //     'keyValue': '',
    //     'pattern': 'regex...',
    //   },
    //   {
    //     'id': '2',
    //     'metaInputType': {
    //       'id': '2',
    //       'name': 'Radio',
    //     },
    //     'name': 'AuthType',
    //     'required': 'true',
    //     'defaultValue': '',
    //     'keyName': 'AuthType',
    //     'keyValue': 'MD5|SHA',
    //     'pattern': 'regex...',
    //   },
    // ];
    console.log('init');
  }

  public handleVerify(): void {
    this.setState({
      isVerifying: true,
    });
    let sd = this.props.getSensor();
    sd.crawlerAuth = this.state.authJson;
    this.props.setSensor(sd);

    this.props.onVerifyCrawler(sd.infra.id, sd.crawler, sd.crawlerAuth);

    let authCrawler: AuthCrawler = {
      crawler: { id: sd.crawler.id },
      target: { id: sd.infra.target.id },
      authJson: JSON.stringify(sd.crawlerAuth),
    };

    this.props.onRegistAuthCrawler(authCrawler);
  }

  public onChangeAuth = (key: string, data: string) => {
    let newJson: any = _.clone(this.state.authJson);
    newJson[key] = data;
    this.setState({ authJson: newJson });

    console.log(newJson);
  }

  public renderRow(item: MetaCrawlerInputItem, index: number): JSX.Element {
    let elem = new Array();
    if (item.inputType.name === 'TEXT_TYPE') {
      elem.push(<Input label={item.name} placeholder={item.defaultValue} key={0} onChange={
        (event: React.SyntheticEvent<HTMLInputElement>, data: InputOnChangeData) => {
          this.onChangeAuth(item.name, data.value);
        }} />);
    } else if (item.inputType.name === 'PASSWORD_TYPE') {
      elem.push(<Input label={item.name} type='password' placeholder={item.defaultValue} key={0} onChange={
        (event: React.SyntheticEvent<HTMLInputElement>, data: InputOnChangeData) => {
          this.onChangeAuth(item.name, data.value);
        }} />);
    } else if (item.inputType.name === 'SELECT_TYPE') {
      let itemValues = item.keyValue.split('|');
      let idx = 0;
      elem.push(<Label pointing='below'>{item.name}</Label>);

      if(this.state.authJson[item.name] === undefined) {
         this.onChangeAuth(item.name, item.defaultValue);
      }

      for (let itemValue of itemValues) {
          elem.push(
            <Form.Radio
              key={idx++}
              label={itemValue}
              name={'radioGroup' + String(item.id)}
              value={itemValue}
              checked={this.state.authJson[item.name] ? this.state.authJson[item.name] === itemValue : itemValue === item.defaultValue}
              defaultChecked={itemValue === item.defaultValue ? true : false}
              onChange={
                (event: React.FormEvent<HTMLInputElement>, data: CheckboxProps) => {
                  this.onChangeAuth(item.name, itemValue);
                }
              }
            />);
        }
    }

    return (
      <Table.Row key={index}>
        <Table.Cell collapsing>{item.keyName}</Table.Cell>
        <Table.Cell >
          <Form.Group inline>{elem}</Form.Group>
        </Table.Cell>
      </Table.Row >
    );
  }

  public renderAuth(): JSX.Element[] {

    if (this.props.metaCrawlerInputItemList === undefined) {
      return null;
    }

    let elements: Array<JSX.Element> = new Array();
    let elem: JSX.Element;
    for (let index: number = 0; index < this.props.metaCrawlerInputItemList.length; ++index) {
      elem = this.renderRow(this.props.metaCrawlerInputItemList[index], index);
      elements.push(elem);
    }

    return elements;
  }

  public render(): JSX.Element {
    // if (this.props.crawlerId === null) {
    //   return null;
    // }
    // this.props.onCrawlerReadAllByMetaCrawler(this.props.crawler);

    if (this.props.metaCrawlerInputItemList === undefined || this.props.metaCrawlerInputItemList.length <= 0) {
      return null;
    }


    return (
      <Segment style={{ width: '350px' }}>
        <Table basic='very' celled >
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell colSpan='2'>{this.props.crawler} 접속 정보</Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {this.renderAuth()}
          </Table.Body>

          <Table.Footer>
            <Table.Row>
              <Table.HeaderCell colSpan='2'>
                {/*loading={this.state.isVerifying}*/}
                <Button primary floated={'right'} onClick={this.handleVerify.bind(this)} loading={this.state.isVerifying}>Verify</Button>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Footer>
        </Table>
      </Segment>
    );
  }
}
