import * as React from 'react';
import { Button, Label } from 'semantic-ui-react';
import {
    BarChart, Bar, XAxis, YAxis, ZAxis, Cell, CartesianGrid, Tooltip, Legend, LineChart, Line,
    ResponsiveContainer, AreaChart, Area, PieChart, Pie, ReferenceLine, Brush, ComposedChart,
} from 'recharts';

export interface StateProps {
}

export interface DispatchProps {
}

export type Props = StateProps & DispatchProps;

export interface State {
    data: any;
}

export class Metric extends React.Component<Props, State> {

    private loadInterval: any;

    constructor(props: Props, context: State) {
        super(props, context);
        this.state = {
            data: undefined,
        };
    }

    public componentWillMount(): void {
        const data = [];
        for (let i = 0; i < 10; i++) {
            let user = Math.floor(Math.random() * 10000) + 1000;
            let nice = Math.floor(Math.random() * 10000) + 1000;
            let system = Math.floor(Math.random() * 10000) + 1000;
            let idle = Math.floor(Math.random() * 10000) + 1000;
            let usage = Math.floor(Math.random() * 100) + 0;

            let elem = { 'id': i, 'user': user, 'nice': nice, 'system': system, 'idle': idle, 'usage': usage };
            data[i] = elem;
        }

        this.setState({
            data: data,
        });
    }

    private showTooltipData = (data: any) => {
        if (data.active && data.payload) {
            return (
                <div style={{ backgroundColor: 'white', opacity: 0.7, border: '1px solid black', padding: '4px' }}>
                    <p><b>{`Total Usage : ${data.payload[0].value}%`}</b></p>
                    <p>{`User : ${data.payload[0].payload.user}`}</p>
                    <p>{`Nice : ${data.payload[0].payload.nice}`}</p>
                    <p>{`System : ${data.payload[0].payload.system}`}</p>
                    <p>{`Idle : ${data.payload[0].payload.idle}`}</p>
                </div>
            );
        }
        return null;
    }


    public render(): JSX.Element {

        const chartData = [
            { name: 'Used', value: 80, fill: '#8884d8', unit: '%' },
            { name: 'Free', value: 20, fill: '#82ca9d', unit: '%' },
        ];


        return (
            <div>

                <AreaChart syncId='anyId' width={800} height={300}
                    data={this.state.data} margin={{ top: 5, right: 30, left: 20, bottom: 5 }} >
                    <XAxis dataKey='id' />
                    <YAxis type='number' domain={[0, 100]} unit='%' />
                    {/* <CartesianGrid stroke='#ccc' /> */}
                    <CartesianGrid strokeDasharray='5 5' />
                    <Tooltip content={this.showTooltipData.bind(this)} />
                    <Legend />
                    <Area type='monotone' dataKey='usage' stroke='#8884d8' fill='#82ca9d' unit='%' name='Usage' activeDot={{ r: 7 }} />
                    <Brush startIndex={3} height={30} />
                </AreaChart>

                <LineChart syncId='anyId' width={800} height={300}
                    data={this.state.data} margin={{ top: 5, right: 30, left: 20, bottom: 5 }} >
                    <XAxis dataKey='id' />
                    <YAxis />
                    {/* <CartesianGrid stroke='#ccc' /> */}
                    <CartesianGrid strokeDasharray='3 3' />
                    <Tooltip />
                    <Legend />
                    <ReferenceLine y={9000} label='Warning' stroke='red' />
                    <Line type='monotone' dataKey='user' stroke='#8884d8' unit='jiffies' activeDot={{ r: 8 }} />
                    <Line type='monotone' dataKey='idle' stroke='#42454a' unit='jiffies' activeDot={{ r: 8 }} />
                </LineChart>

                <BarChart syncId='anyId' width={800} height={250} data={this.state.data}>
                    <XAxis dataKey='id' />
                    <YAxis />
                    <CartesianGrid strokeDasharray='3 3' />
                    <Tooltip />
                    <Legend />
                    <Bar dataKey='user' fill='#8884d8' unit='jiffies' />
                    <Bar dataKey='idle' fill='#82ca9d' unit='jiffies' />
                </BarChart>

                <ComposedChart width={800} height={400} data={this.state.data}>
                    <XAxis dataKey='id' />
                    <YAxis />
                    <CartesianGrid stroke='#f5f5f5' />
                    <Tooltip />
                    <Legend />
                    <Bar dataKey='user' fill='#ffc658' unit='jiffies' stackId='mixed' />
                    <Bar dataKey='idle' fill='#82ca9d' unit='jiffies' stackId='mixed' />
                    <Bar dataKey='nice' fill='#98ca41' unit='jiffies' stackId='mixed' />
                    <Line dataKey='nice' type='monotone' stroke='#ff7300' unit='jiffies' />
                </ComposedChart>

                <PieChart width={800} height={400}>
                    <text fontSize='20px' x='50%' y='50%' dy={8} textAnchor='middle' fill='#8884d8'>
                         {chartData[0].value}%
                    </text>
                    <Pie data={chartData}
                        cx='50%'
                        cy='50%'
                        outerRadius={80}
                        innerRadius={60}
                        startAngle={90}
                        endAngle={-270}
                    />

                </PieChart>


            </div>
        );
    }

}


