import * as React from 'react';

import {
  Icon,
  Step,
  Button,
  Table,
  Radio,
  Form,
  Container,
  Checkbox,
} from 'semantic-ui-react';
import {
  Grid,
  Image,
  Label,
  Segment,
  Dropdown,
  Input,
  List,
  Accordion,
  Loader,
  DropdownItemProps, DropdownProps,
} from 'semantic-ui-react';


import MetaCrawler from '@overflow/meta/api/model/MetaCrawler';
import Infra from '@overflow/infra/api/model/Infra';
import Domain from '@overflow/domain/api/model/Domain';

import Page from '@overflow/commons/api/model/Page';
import SensorRegistInfo from '../../api/model/SensorRegistInfo';

import PageParams from '@overflow/commons/api/model/PageParams';

import * as Utils from '@overflow/commons/util/Utils';

export interface StateProps {
  infraList?: Page;
  infra?: Infra;
  infraId?: number;
  setSensor?(data: SensorRegistInfo): void;
  getSensor?(): SensorRegistInfo;
  stepNext?(): void;
}

export interface DispatchProps {

  onReadAllTargetByDomain?(domain: Domain, pageParams: PageParams): void;
  onReadInfra?(infraId: number): void;
}

export type Props = StateProps & DispatchProps;

export interface State {
  selected: Infra;
}



export class SensorConfigTargetSelect extends React.Component<Props, State> {

  private selectOptions: Array<DropdownItemProps>;

  constructor(props: Props, context: State) {
    super(props, context);
    this.state = {
      selected: null,
    };
  }

  public componentWillMount(): void {
    if (this.props.infraId === undefined) {
      const pageParams: PageParams = {
        pageNo: '0',
        countPerPage: '999',
        sortCol: 'id',
        sortDirection: 'descending',
      };
      this.props.onReadAllTargetByDomain({ id: 1 }, pageParams);
    } else {
      this.props.onReadInfra(this.props.infraId);
    }
  }

  public handleSelect(infra: Infra): void {
    this.setState({
      selected: infra,
    });

    let sd = this.props.getSensor();
    sd.infra = infra;
    this.props.setSensor(sd);
  }

  public handleRowActive(infra: Infra): boolean {
    if (this.state.selected === infra) {
      return true;
    }
    return false;
  }

  public convertInfraList(): JSX.Element[] {

    if (this.props.infraList === undefined) {
      return null;
    }

    let elems: Array<JSX.Element> = new Array();

    for (let index: number = 0; index < this.props.infraList.content.length; ++index) {
      let infra: Infra = this.props.infraList.content[index];
      elems.push(
        <Table.Row key={infra.id} onClick={this.handleSelect.bind(this, infra)} active={this.handleRowActive(infra)}>
          <Table.Cell textAlign={'center'}>{index + 1}</Table.Cell>
          <Table.Cell textAlign={'center'}>{infra.infraType.name}</Table.Cell>
          <Table.Cell textAlign={'center'}>{infra.target.displayName}</Table.Cell>
          <Table.Cell>TODO</Table.Cell>
          <Table.Cell>{Utils.date2date(infra.target.createDate)}</Table.Cell>
        </Table.Row>,
      );

    }

    return elems;
  }

  public convertInfra(): void {
    if (this.props.infra === undefined) {
      return;
    }
    let selectionOptions: Array<DropdownItemProps> = new Array;

    selectionOptions.push(this.createOption(this.props.infra));
    this.selectOptions = selectionOptions;

    let sd = this.props.getSensor();
    // sd.targetId = this.props.infra.target.id;
    // sd.type = this.props.infra.infraType.name;
    sd.infra = this.props.infra;
    this.props.setSensor(sd);
    this.props.stepNext();
  }

  public createOption(infra: Infra): DropdownItemProps {
    let option = {
      key: infra.id,
      text: infra.infraType.name + '-' + infra.target.displayName,
      // value: infra.target.id,
      value: JSON.stringify(infra),
      // icon: 'check', // or close?
    };
    return option;
  }

  public defaultValue(): any {
    if (this.selectOptions !== undefined && this.selectOptions.length > 0) {
      return this.selectOptions[0].value;
    }
    return null;
  }

  public onChangeTarget(event: React.SyntheticEvent<HTMLElement>, data: DropdownProps): void {
    let sd = this.props.getSensor();
    // let infra: Infra = JSON.parse(String(data.value));
    // sd.targetId = infra.target.id;
    // sd.type = infra.infraType.name;
    sd.infra = JSON.parse(String(data.value));
    this.props.setSensor(sd);
  }

  public renderInfra(): JSX.Element[] {

    if (this.props.infraId === undefined) {
      return this.convertInfraList();
    } else {
      this.convertInfra();
    }

    return null;

    // return (<Dropdown
    //   placeholder='Select Target' selection options={this.selectOptions}
    //   // defaultValue={this.defaultValue()}
    //   onChange={this.onChangeTarget.bind(this)} />);
  }

  public render(): JSX.Element {
    return (
      <Container fluid>
        {/* */}
        <Table celled selectable>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell textAlign={'center'}>No.</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Type</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Name</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Sensor Count</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created at</Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {this.renderInfra()}
          </Table.Body>
        </Table>
      </Container>
    );
  }
}
